import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;

/**
 * Class CellRenderer
 * It renders a cell's content.
 */
public class CellRenderer {

	FieldCell cell;
	int posX, posY;

	public CellRenderer(FieldCell c, int x, int y){
		cell = c;
		posX = x;
		posY = y;
	}

	public void render(Graphics g, RenderTarget rt){
		int image = Enum.G_EMPTY;
		int rotate = 0;
		int type =  cell.getContentType();
		int od, nd;
		SnakeDead sd;
		SnakeBody sb;
		SnakeHead sh;
		switch (type){
			// Draws Empty
			case Enum.UNKNOWN:
			case Enum.EMPTY:
				image = Enum.G_EMPTY;
				break;
			// Draws a Wall
			case Enum.WALL:
				image = Enum.G_WALL;
				break;
			// Draws a FieldBerry
			case Enum.FIELDBERRY:
				image = Enum.G_FIELDBERRY;
				break;
			// Draws a SawBerry
			case Enum.SAWBERRY:
				image = Enum.G_SAWBERRY;
				break;
			// Draws a StoneBerry
			case Enum.STONEBERRY:
				image =  Enum.G_STONEBERRY;
				break;
			// Draws a dead SnakeHead
			case Enum.DEADSNAKEHEAD:
				sd = (SnakeDead)cell.getContent();
				// Head ends with no dead bodyparts
				if (!sd.hasNext()){
					rotate = getLinearRotation(sd.getDirection());
					// Head has stone
					if (sd.getStoneStatus())
						image = Enum.G_SNAKE_DEAD_HEAD_END_STONE;
					// Head has no stone
					else
						image = Enum.G_SNAKE_DEAD_HEAD_END;
				}
				// Head has more dead bodyparts
				else{
					rotate = getLinearRotation(sd.getNextDirection());
					// Head has stone
					if (sd.getStoneStatus())
						image = Enum.G_SNAKE_DEAD_HEAD_STONE;
					// Head has no stone
					else
						image = Enum.G_SNAKE_DEAD_HEAD;
				}
				break;
			// Draws a dead SnakeBody
			case Enum.DEADSNAKEBODY:
				sd = (SnakeDead)cell.getContent();
				// Dead snake ends with no dead bodyparts
				if (!sd.hasNext()){
					rotate = getLinearRotation(sd.getDirection());
					// Bodypart has stone
					if (sd.getStoneStatus())
						image = Enum.G_SNAKE_DEAD_END_STONE;
					// Bodypart has no stone
					else
						image = Enum.G_SNAKE_DEAD_END;
				}
				// Dead snake has more dead bodyparts
				else{
					od = sd.getDirection();
					nd = sd.getNextDirection();
					// Snake's bodypart is linear
					if (od == nd){
						rotate = getLinearRotation(od);
						// Head has stone
						if (sd.getStoneStatus())
							image = Enum.G_SNAKE_DEAD_LINEAR_STONE;
						// Head has no stone
						else
							image = Enum.G_SNAKE_DEAD_LINEAR;
					}
					// Snake's bodypart is bent
					else{
						switch (od){
							case Enum.UP:
								if (nd == Enum.RIGHT)
									rotate = 180;
								else // LEFT
									rotate = 270;
								break;
							case Enum.RIGHT:
								if (nd == Enum.UP)
									rotate = 0;
								else // DOWN
									rotate = 270;
								break;
							case Enum.DOWN:
								if (nd == Enum.RIGHT)
									rotate = 90;
								else // LEFT
									rotate = 0;
								break;
							case Enum.LEFT:
								if (nd == Enum.UP)
									rotate = 90;
								else  // DOWN
									rotate = 180;
								break;
						}
						// Body has stone
						if (sd.hasStone)
							image = Enum.G_SNAKE_DEAD_BENT_STONE;
						// Body has no stone
						else
							image = Enum.G_SNAKE_DEAD_BENT;
					}
				}
				break;
			// Draws a SnakeHead
			case Enum.SNAKEHEAD:
				sh = (SnakeHead)cell.getContent();
				// Head ends with no bodyparts
				if (!sh.hasNext()){
					rotate = getLinearRotation(sh.getDirection());
					// Head has stone
					if (sh.getStoneStatus()){
						// Snake is berserked
						if (SnakeState.isBerserked(sh.getSnake().getState())){
							// Berserk Time
							int bt = sh.getSnake().getBerserkTime();
							// Flashing interval
							int fi  = Enum.SNAKEMOVETIME*6;
							int ft1 = Enum.SNAKEMOVETIME;
							int ft2 = Enum.SNAKEMOVETIME*2;
							switch(sh.getSnake().getController()){
								case Enum.PLAYER1:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_PL1_HEAD_END_STONE;
									else
										image = Enum.G_SNAKE_PL1_HEAD_END_STONE_BERSERKED;
									break;
								case Enum.PLAYER2:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_PL2_HEAD_END_STONE;
									else
										image = Enum.G_SNAKE_PL2_HEAD_END_STONE_BERSERKED;
									break;
								case Enum.PLAYER3:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_PL3_HEAD_END_STONE;
									else
										image = Enum.G_SNAKE_PL3_HEAD_END_STONE_BERSERKED;
									break;
								case Enum.PLAYER4:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_PL4_HEAD_END_STONE;
									else
										image = Enum.G_SNAKE_PL4_HEAD_END_STONE_BERSERKED;
									break;
								case Enum.NOCONTROLLER:
								case Enum.PLAYERAI:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_AI_HEAD_END_STONE;
									else
										image = Enum.G_SNAKE_AI_HEAD_END_STONE_BERSERKED;
									break;
							}
						}
						// Snake is not berserked
						else{
							switch(sh.getSnake().getController()){
								case Enum.PLAYER1:
									image = Enum.G_SNAKE_PL1_HEAD_END_STONE;
									break;
								case Enum.PLAYER2:
									image = Enum.G_SNAKE_PL2_HEAD_END_STONE;
									break;
								case Enum.PLAYER3:
									image = Enum.G_SNAKE_PL3_HEAD_END_STONE;
									break;
								case Enum.PLAYER4:
									image = Enum.G_SNAKE_PL4_HEAD_END_STONE;
									break;
								case Enum.NOCONTROLLER:
								case Enum.PLAYERAI:
									image = Enum.G_SNAKE_AI_HEAD_END_STONE;
									break;
							}
						}
					}
					// Head has no stone
					else{
						// Snake is berserked
						if (SnakeState.isBerserked(sh.getSnake().getState())){
							// Berserk Time
							int bt = sh.getSnake().getBerserkTime();
							// Flashing interval
							int fi  = Enum.SNAKEMOVETIME*6;
							int ft1 = Enum.SNAKEMOVETIME;
							int ft2 = Enum.SNAKEMOVETIME*2;
							switch(sh.getSnake().getController()){
								case Enum.PLAYER1:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_PL1_HEAD_END;
									else
										image = Enum.G_SNAKE_PL1_HEAD_END_BERSERKED;
									break;
								case Enum.PLAYER2:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_PL2_HEAD_END;
									else
										image = Enum.G_SNAKE_PL2_HEAD_END_BERSERKED;
									break;
								case Enum.PLAYER3:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_PL3_HEAD_END;
									else
										image = Enum.G_SNAKE_PL3_HEAD_END_BERSERKED;
									break;
								case Enum.PLAYER4:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_PL4_HEAD_END;
									else
										image = Enum.G_SNAKE_PL4_HEAD_END_BERSERKED;
									break;
								case Enum.NOCONTROLLER:
								case Enum.PLAYERAI:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_AI_HEAD_END;
									else
										image = Enum.G_SNAKE_AI_HEAD_END_BERSERKED;
									break;
							}
						}
						// Snake is not berserked
						else{
							switch(sh.getSnake().getController()){
								case Enum.PLAYER1:
									image = Enum.G_SNAKE_PL1_HEAD_END;
									break;
								case Enum.PLAYER2:
									image = Enum.G_SNAKE_PL2_HEAD_END;
									break;
								case Enum.PLAYER3:
									image = Enum.G_SNAKE_PL3_HEAD_END;
									break;
								case Enum.PLAYER4:
									image = Enum.G_SNAKE_PL4_HEAD_END;
									break;
								case Enum.NOCONTROLLER:
								case Enum.PLAYERAI:
									image = Enum.G_SNAKE_AI_HEAD_END;
									break;
							}
						}
					}
				}
				// Head has more bodyparts
				else{
					rotate = getLinearRotation(sh.getNext().getDirection());
/*					for (int i=0; i<Enum.NUMNEIGHBOURS; i++){
						SnakeBody next = sh.getNext();
						IContent tmp = next.getPosition().getNeighbour(i).getContent();
						if (tmp.getType() == Enum.SNAKEBODY && ((SnakeBody)tmp).equals(sh))
							rotate = getLinearRotation(i);
					}*/
					// Head has stone
					if (sh.getStoneStatus()){
						// Snake is berserked
						if (SnakeState.isBerserked(sh.getSnake().getState())){
							// Berserk Time
							int bt = sh.getSnake().getBerserkTime();
							// Flashing interval
							int fi  = Enum.SNAKEMOVETIME*6;
							int ft1 = Enum.SNAKEMOVETIME;
							int ft2 = Enum.SNAKEMOVETIME*2;
							switch(sh.getSnake().getController()){
								case Enum.PLAYER1:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_PL1_HEAD_STONE;
									else
										image = Enum.G_SNAKE_PL1_HEAD_STONE_BERSERKED;
									break;
								case Enum.PLAYER2:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_PL2_HEAD_STONE;
									else
										image = Enum.G_SNAKE_PL2_HEAD_STONE_BERSERKED;
									break;
								case Enum.PLAYER3:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_PL3_HEAD_STONE;
									else
										image = Enum.G_SNAKE_PL3_HEAD_STONE_BERSERKED;
									break;
								case Enum.PLAYER4:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_PL4_HEAD_STONE;
									else
										image = Enum.G_SNAKE_PL4_HEAD_STONE_BERSERKED;
									break;
								case Enum.NOCONTROLLER:
								case Enum.PLAYERAI:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_AI_HEAD_STONE;
									else
										image = Enum.G_SNAKE_AI_HEAD_STONE_BERSERKED;
									break;
							}
						}
						// Snake is not berserked
						else{
							switch(sh.getSnake().getController()){
								case Enum.PLAYER1:
									image = Enum.G_SNAKE_PL1_HEAD_STONE;
									break;
								case Enum.PLAYER2:
									image = Enum.G_SNAKE_PL2_HEAD_STONE;
									break;
								case Enum.PLAYER3:
									image = Enum.G_SNAKE_PL3_HEAD_STONE;
									break;
								case Enum.PLAYER4:
									image = Enum.G_SNAKE_PL4_HEAD_STONE;
									break;
								case Enum.NOCONTROLLER:
								case Enum.PLAYERAI:
									image = Enum.G_SNAKE_AI_HEAD_STONE;
									break;
							}
						}
					}
					// Head has no stone
					else{
						// Snake is berserked
						if (SnakeState.isBerserked(sh.getSnake().getState())){
							// Berserk Time
							int bt = sh.getSnake().getBerserkTime();
							// Flashing interval
							int fi  = Enum.SNAKEMOVETIME*6;
							int ft1 = Enum.SNAKEMOVETIME;
							int ft2 = Enum.SNAKEMOVETIME*2;
							switch(sh.getSnake().getController()){
								case Enum.PLAYER1:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_PL1_HEAD;
									else
										image = Enum.G_SNAKE_PL1_HEAD_BERSERKED;
									break;
								case Enum.PLAYER2:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_PL2_HEAD;
									else
										image = Enum.G_SNAKE_PL2_HEAD_BERSERKED;
									break;
								case Enum.PLAYER3:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_PL3_HEAD;
									else
										image = Enum.G_SNAKE_PL3_HEAD_BERSERKED;
									break;
								case Enum.PLAYER4:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_PL4_HEAD;
									else
										image = Enum.G_SNAKE_PL4_HEAD_BERSERKED;
									break;
								case Enum.NOCONTROLLER:
								case Enum.PLAYERAI:
									if (bt<fi && bt%ft2>ft1)
										image = Enum.G_SNAKE_AI_HEAD;
									else
										image = Enum.G_SNAKE_AI_HEAD_BERSERKED;
									break;
							}
						}
						// Snake is not berserked
						else{
							switch(sh.getSnake().getController()){
								case Enum.PLAYER1:
									image = Enum.G_SNAKE_PL1_HEAD;
									break;
								case Enum.PLAYER2:
									image = Enum.G_SNAKE_PL2_HEAD;
									break;
								case Enum.PLAYER3:
									image = Enum.G_SNAKE_PL3_HEAD;
									break;
								case Enum.PLAYER4:
									image = Enum.G_SNAKE_PL4_HEAD;
									break;
								case Enum.NOCONTROLLER:
								case Enum.PLAYERAI:
									image = Enum.G_SNAKE_AI_HEAD;
									break;
							}
						}
					}
				}
				break;
			// Draws a SnakeBody
			case Enum.SNAKEBODY:
				sb = (SnakeBody)cell.getContent();
				// Snake ends with no bodyparts
				if (!sb.hasNext()){
					rotate = getLinearRotation(sb.getDirection());
					// Bodypart has stone
					if (sb.getStoneStatus()){
						switch(sb.getSnake().getController()){
							case Enum.PLAYER1:
								image = Enum.G_SNAKE_PL1_END_STONE;
								break;
							case Enum.PLAYER2:
								image = Enum.G_SNAKE_PL2_END_STONE;
								break;
							case Enum.PLAYER3:
								image = Enum.G_SNAKE_PL3_END_STONE;
								break;
							case Enum.PLAYER4:
								image = Enum.G_SNAKE_PL4_END_STONE;
								break;
							case Enum.NOCONTROLLER:
							case Enum.PLAYERAI:
								image = Enum.G_SNAKE_AI_END_STONE;
								break;
						}
					}
					// Bodypart has no stone
					else{
						switch(sb.getSnake().getController()){
							case Enum.PLAYER1:
								image = Enum.G_SNAKE_PL1_END;
								break;
							case Enum.PLAYER2:
								image = Enum.G_SNAKE_PL2_END;
								break;
							case Enum.PLAYER3:
								image = Enum.G_SNAKE_PL3_END;
								break;
							case Enum.PLAYER4:
								image = Enum.G_SNAKE_PL4_END;
								break;
							case Enum.NOCONTROLLER:
							case Enum.PLAYERAI:
								image = Enum.G_SNAKE_AI_END;
								break;
						}
					}
				}
				// Snake has more bodyparts
				else{
					od = sb.getDirection();
					nd = sb.getNext().getDirection();
					// Snake's bodypart is linear
					if (od == nd){
						rotate = getLinearRotation(od);
						// Head has stone
						if (sb.getStoneStatus()){
							switch(sb.getSnake().getController()){
								case Enum.PLAYER1:
									image = Enum.G_SNAKE_PL1_LINEAR_STONE;
									break;
								case Enum.PLAYER2:
									image = Enum.G_SNAKE_PL2_LINEAR_STONE;
									break;
								case Enum.PLAYER3:
									image = Enum.G_SNAKE_PL3_LINEAR_STONE;
									break;
								case Enum.PLAYER4:
									image = Enum.G_SNAKE_PL4_LINEAR_STONE;
									break;
								case Enum.NOCONTROLLER:
								case Enum.PLAYERAI:
									image = Enum.G_SNAKE_AI_LINEAR_STONE;
									break;
							}
						}
						// Head has no stone
						else{
							switch(sb.getSnake().getController()){
								case Enum.PLAYER1:
									image = Enum.G_SNAKE_PL1_LINEAR;
									break;
								case Enum.PLAYER2:
									image = Enum.G_SNAKE_PL2_LINEAR;
									break;
								case Enum.PLAYER3:
									image = Enum.G_SNAKE_PL3_LINEAR;
									break;
								case Enum.PLAYER4:
									image = Enum.G_SNAKE_PL4_LINEAR;
									break;
								case Enum.NOCONTROLLER:
								case Enum.PLAYERAI:
									image = Enum.G_SNAKE_AI_LINEAR;
									break;
							}
						}
					}
					// Snake's bodypart is bent
					else{
						switch (od){
							case Enum.UP:
								if (nd == Enum.RIGHT)
									rotate = 180;
								else // LEFT
									rotate = 270;
								break;
							case Enum.RIGHT:
								if (nd == Enum.UP)
									rotate = 0;
								else // DOWN
									rotate = 270;
								break;
							case Enum.DOWN:
								if (nd == Enum.RIGHT)
									rotate = 90;
								else // LEFT
									rotate = 0;
								break;
							case Enum.LEFT:
								if (nd == Enum.UP)
									rotate = 90;
								else  // DOWN
									rotate = 180;
								break;
						}
						// Body has stone
						if (sb.hasStone){
							switch(sb.getSnake().getController()){
								case Enum.PLAYER1:
									image = Enum.G_SNAKE_PL1_BENT_STONE;
									break;
								case Enum.PLAYER2:
									image = Enum.G_SNAKE_PL2_BENT_STONE;
									break;
								case Enum.PLAYER3:
									image = Enum.G_SNAKE_PL3_BENT_STONE;
									break;
								case Enum.PLAYER4:
									image = Enum.G_SNAKE_PL4_BENT_STONE;
									break;
								case Enum.NOCONTROLLER:
								case Enum.PLAYERAI:
									image = Enum.G_SNAKE_AI_BENT_STONE;
									break;
							}
						}
						// Body has no stone
						else{
							switch(sb.getSnake().getController()){
								case Enum.PLAYER1:
									image = Enum.G_SNAKE_PL1_BENT;
									break;
								case Enum.PLAYER2:
									image = Enum.G_SNAKE_PL2_BENT;
									break;
								case Enum.PLAYER3:
									image = Enum.G_SNAKE_PL3_BENT;
									break;
								case Enum.PLAYER4:
									image = Enum.G_SNAKE_PL4_BENT;
									break;
								case Enum.NOCONTROLLER:
								case Enum.PLAYERAI:
									image = Enum.G_SNAKE_AI_BENT;
									break;
							}
						}
					}
				}
				break;
		}

		BufferedImage im = rt.getContentImage(image);
		double tx = Enum.G_PIXEL/2.0;
		double ty = Enum.G_PIXEL/2.0;
		if (im != null && g != null){

			AffineTransform tr = new AffineTransform();
			Graphics2D g2d = (Graphics2D) g;
			// S3: final translation
			tr.translate(posX+tx+Enum.G_TRANSLATEX, posY+ty+Enum.G_TRANSLATEY);
			// S2: rotate around anchor
			tr.rotate(Math.toRadians(rotate));
			// S1: translate anchor to origin
			tr.translate(-tx, -ty);

			AffineTransform saveAT = g2d.getTransform();
			if (g2d != null){
				g2d.drawImage(im, tr, null); 
				g2d.setTransform(saveAT);
			}
		}
	}

	private int getLinearRotation(int dir){
		switch (dir){
			case Enum.UP:
				return 0;
			case Enum.RIGHT:
				return 90;
			case Enum.DOWN:
				return 180;
			case Enum.LEFT:
				return 270;
			default: return 0;
		}
	}



}
