 import java.util.Random;

/**
 * Class Field
 * Represent the Field of the game, where the
 * entities interacts with each other.
 * It generates the contents of the FieldCells,
 * it contains all the entities that exists.
 * It uses a 2d map with every
 * cell having exactly 4 neighbours.
 */
public class Field {

	// references to the cells
	private FieldCell[] cells;

	// references to the snakes
	private Snake[] snakes;

	// reference to the Game object
	public Game game;

	// width of the Field
	public int fieldWidth = 10;

	// height of the Field
	public int fieldHeight = 10;

	// number of Cells on the Field
	public int numCells = fieldWidth * fieldHeight;	

	// number of Neihgbours of each Cell
	private int numNeighbours = Enum.NUMNEIGHBOURS;

	//number of Snakes on the Field for the Skeleton
	private int numSnakes;

	// Wraparound: on => true or off => false
	public boolean wraparound = true;



/**
 * Class constructor.
 */
	public Field(Game g){
		game = g;
	}



/**
 * Resets the class.
 */
	public void resetField(){
		cells = null;
		snakes = null;
		numSnakes = 0;
		numCells = 0;
		wraparound = true;

		// IDs reseted
		Snake.idnum = 0;
		SnakeHead.idnum = 0;
		SnakeBody.idnum = 0;
		SnakePart.idnum = 0;
		Berry.idnum = 0;
		SnakeDead.idnum = 0;
	}



/**
 * Generates the field.
 */
   	public void generateField(int width, int height, boolean wrap, float wallpercentage, String method, int players, AIPlayer aiplayer[], HumanPlayer[] huplayer, Renderer renderer){

		// Validates width and height
		fieldWidth = validateWidth(width);
		fieldHeight = validateHeight(height);
		numCells = fieldWidth * fieldHeight;

		// Sets walls
		for (int i=0; i<10; i++){
			// Rests the current Field;
			resetField();

			// Sets field
			generateEmptyField(fieldWidth, fieldHeight);

			// Sets wraparound
			if (!wrap)
				wraparoundOFF();
			wraparound = wrap;

			if (addWallPercentage(wallpercentage, method)){
				//System.out.println("<< Map generated successfully.");
				i = 10;
			}
			else{
				//System.out.println("<< Error! Invalid map generated. Please ty again.");
			}
		}

		// Sets snakes
		numSnakes = players;
		Snake.idnum = 0;
		SnakeHead.idnum = 0;
		SnakeBody.idnum = 0;
		SnakeDead.idnum = 0;
		SnakePart.idnum = 0;
		snakes = new Snake[numSnakes];

		// Snakes are put on the field.
		Random r = new Random();
		for (int i=0; i<numSnakes; i++){
			int tries = 0;
			snakes[i] = new Snake();
			while(/*tries != 20*/true){
				int x = r.nextInt(fieldWidth);
				int y = r.nextInt(fieldHeight);
				int dir = r.nextInt(numNeighbours);
				if (!addSnakeToField(x, y, Enum.SNAKELENGTH, dir, snakes[i])){
					tries++;
					//System.out.println("<< ("+tries+") Tries generating the snake again.");
				}
				else{
					System.out.println("Snake["+snakes[i].getID()+"] created after "+tries+" tries.");
					break;
					//tries = 20;
				}
			}
		}
		int s = 0;
		for (int i=0; i<snakes.length; i++){
			if (snakes[i] != null && snakes[i].getHead() != null && snakes[i].getHead().getPosition()!=null)
				s++;
		}
		if (s != snakes.length){
			Snake[] tmp = new Snake[s];
			s = 0;
			for (int i=0; i<snakes.length; i++){
				if (snakes[i] != null && snakes[i].getHead() != null && snakes[i].getHead().getPosition() != null){
					tmp[s]=snakes[i];
					s++;
				}
			System.out.println("Not all snakes could be created.");
			snakes = tmp;
			numSnakes = snakes.length;
		}
		
		}

		// Sets init direction of snakes
		r = new Random();
		for (int c=0; c<numCells; c++){
			if (cells[c].getContentType() == Enum.SNAKEHEAD){
				initDirection((SnakeHead)(cells[c].getContent()));
			}
		}

		// Sets players
		// -- Human players
		if (huplayer != null){
			for (int i=0; i<huplayer.length; i++)
				huplayer[i].setSnake(snakes[i]);
		}
		// -- AI players
		if (aiplayer != null){
			for (int i=0; i<aiplayer.length; i++){
				aiplayer[i].setSnake(snakes[i+huplayer.length]);
				aiplayer[i].setField(get2d());
			}
		}

		// Sets renderer
		renderer.init(fieldWidth, fieldHeight, get2d());

	}



/**
 * Generates the field for testing purposes.
 */
   	public void generateEmptyField(int width, int height){

		// Validates width and height
		width = validateWidth(width);
		height = validateHeight(height);

		// Rests the current Field;
		resetField();

		// Sets new size
		fieldWidth = width;
		fieldHeight = height;
		numCells = width * height;

		// Sets cells
		FieldCell.idnum = 0;
		cells = new FieldCell[numCells];
		for (int i=0; i<numCells; i++)
			cells[i] = new FieldCell(numNeighbours);
		
		// Sets cells' neighbours
		setAllNeighbours();

		// Sets all cells empty
		Empty empty = new Empty();
		for (int i=0; i<numCells; i++){
			cells[i].updateContent(empty);
		}

	}



/**
 * It commands all the snakes to move to the next
 * FieldCell.
 */
	public void moveSnakes(){
		if (snakes != null){
			for (int i=0; i<snakes.length; i++){
				if (snakes[i].alive)
					snakes[i].move();
			}
		}
	}



/**
 * Checks if the value of width is correct.
 * 
 * @param width The width to check.
 */
	private int validateWidth(int width){
		if (width == 0) return fieldWidth;
		if (width < Enum.WIDTHMIN){
			width = Enum.WIDTHMIN;
		}
		else if (width > Enum.WIDTHMAX){
			width = Enum.WIDTHMAX;
		}
		return width;
	}



/**
 * Checks if the value of height is correct.
 * 
 * @param height The height to check.
 */
	private int validateHeight(int height){
		if (height == 0) return fieldHeight;
		if (height < Enum.HEIGHTMIN){
			height = Enum.HEIGHTMIN;
		}
		else if (height > Enum.HEIGHTMAX){
			height = Enum.HEIGHTMAX;
		}
		return height;
	}



/**
 * Sets wraparound off
 */
	public void wraparoundOFF(){
		int tmpwidth  = fieldWidth + 2;
		int tmpheight = fieldHeight + 2;
		int tmpcells  = tmpwidth * tmpheight;
		Wall wall = new Wall();
		FieldCell tmp[];
		tmp = new FieldCell[tmpcells];

		if (wraparound){
			wraparound = false;
			//System.out.println("<< Wraparound is set off.");
			// Wraparound added
			// i: number of cells in the old Field (lesser)
			// j: number of cells in the new Field (greater)

			for (int j=0, i=0; j<tmpcells; j++){
				if ((j >= tmpwidth) && (j <= tmpwidth*(tmpheight-1)) && (j%tmpwidth != 0) && ((j+1)%tmpwidth != 0 )) {
					tmp[j] = cells[i];
					i++;
				}
				else{
					tmp[j] = new FieldCell(4);
					tmp[j].updateContent(wall);
				}
			}
		cells = tmp;
		fieldWidth = tmpwidth;
		fieldHeight = tmpheight;
		numCells = tmpcells;
		setAllNeighbours();
		}
	}



/**
 * Sets wraparound on
 */
	public void wraparoundON(){
		int tmpwidth  = fieldWidth - 2;
		int tmpheight = fieldHeight - 2;
		int tmpcells  = tmpwidth * tmpheight;
		FieldCell tmp[];
		tmp = new FieldCell[tmpcells];

		if (!wraparound){
			wraparound = true;
			//System.out.println("<< Wraparound is set on.");

			// Wraparound removed
			// i: number of cells in the old Field (greater)
			// j: number of cells in the new Field (lesser)

			for (int i=0, j=0; i<numCells; i++){
				if ((i >= fieldWidth) && (i <= fieldWidth*(fieldHeight-1)) && (i%fieldWidth != 0) && ((i+1)%fieldWidth != 0 )){
					tmp[j] = cells[i];
					j++;
				}
			}
		cells = tmp;
		fieldWidth = tmpwidth;
		fieldHeight = tmpheight;
		numCells = tmpcells;
		setAllNeighbours();
		}
	}




/**
 * Creates a 2d map of the field and sets the neighbours.
 */
	protected void setAllNeighbours(){

		// Every cells having exactly 4 neighbours.
		for (int i=0; i<numCells; i++){
			for (int l=0; l<numNeighbours; l++)
				cells[i].setNeighbour(l, getNeighbour(i, l));
		}
	}


/**
 * Get neighbour
 *
 * @param num Number of FieldCell
 * @param dir Gives neighbour in this direction
 */
	public FieldCell getNeighbour(int num, int dir){

		// Position of num in a 2d field
		int w = num%fieldWidth;
		int h = num/fieldWidth;

		// Shows the field in 2d map
		FieldCell[][] tmp = get2d();

		switch(dir){
			case Enum.UP:    return tmp[w][((h-1)+fieldHeight)%fieldHeight];
			case Enum.RIGHT: return tmp[(w+1)%fieldWidth][h];
			case Enum.DOWN:  return tmp[w][(h+1)%fieldHeight];
			case Enum.LEFT:  return tmp[((w-1)+fieldWidth)%fieldWidth][h];
			default:         return null;
		}
	}


/**
 * Get cellID
 */
	protected int getCellID(FieldCell c){
		if (c != null){
			for (int i=0; i<numCells; i++){
				if (c.equals(cells[i])){
					return i;
				}
			}
		}
		return -1;
	}



/**
 * Draw Field
 */
	public void drawField(){
		 Draw.drawField(cells, fieldWidth, fieldHeight);
	}



/**
 * Adds walls to the fiels and checks if it is valid or not.
 *
 * @param wallpercentage Amount of walls to add.
 * @param method Method of generating the walls.
 */
	public boolean addWallPercentage(float wallpercentage, String method){

		//System.out.println("<< Wallpercentage:");

		FieldCell[][] tmp;
		FieldCell[] ltmp;
		int tmpwidth;
		int tmpheight;
		int tmpcells;

		if (wallpercentage > 10){
			//System.out.println("<< \tWarning! Wallpercentage exceeeds 10. Set to 10.");
			wallpercentage = 10.0f;
		}
		if (wallpercentage < 0){
			//System.out.println("<< \tWarning! Wallpercentage is smaller than 0. Set to 0.");
			wallpercentage = 0.0f;
		}

		int walls = (int)(numCells * wallpercentage / 100);

		//System.out.println("<< \tNumber of walls needed: " + walls);
		int wallexist = 0;
		for (int i=0; i<cells.length; i++){
			if (cells[i].getContentType() == Enum.WALL)
				wallexist++;
		}
		//System.out.println("<< \tNumber of walls exist: " + wallexist);

		if (!wraparound){
			wallexist = wallexist - (2*fieldHeight) - (2*fieldWidth) + 4;
			//System.out.println("<< \tNumber of walls exist without border: " + wallexist);
		}
		walls -= wallexist;
		//System.out.println("<< \tNumber of walls needs to be added: " + walls);

		// Wraparound is off: field shrinked
		if (!wraparound){
			tmpwidth = fieldWidth - 2;
			tmpheight = fieldHeight - 2;
			tmpcells  = tmpwidth * tmpheight;
			ltmp = new FieldCell[tmpcells];
			for (int i=0, j=0; i<numCells; i++){
				if ((i >= fieldWidth) && (i <= fieldWidth*(fieldHeight-1)) && (i%fieldWidth != 0) && ((i+1)%fieldWidth != 0 )){
					ltmp[j] = cells[i];
					j++;
				}
			}
		}
		// Wraparound is on: field not shrinked
		else{
			ltmp = cells;
			tmpwidth = fieldWidth;
			tmpheight = fieldHeight;
			tmpcells = numCells;
		}

		// TMP set from linear TMP
		tmp = new FieldCell[tmpwidth][tmpheight];
		for (int k=0, i=0; k<tmpheight; k++){
			for (int j=0; j<tmpwidth; j++){
				tmp[j][k] = ltmp[i];
				i++;
			}
		}

		if (method.equals("random")){
			// Walls randomly created
			Random r = new Random();
			Wall wall = new Wall();
			if (walls > 0){
				for (int i=0; i<walls; ){
					int n = r.nextInt(tmpcells);
					if (ltmp[n].getContentType() != Enum.WALL){
						ltmp[n].updateContent(wall);
						i++;
					}
				}
			}
		}
		else if (method.equals("room")){
			// Walls created to form roomlike parts
			Random r = new Random();
			Wall wall = new Wall();
			if (walls > 0){
				for (int i=0; i<walls; ){
					int nw = r.nextInt(tmpwidth);
					int nh = r.nextInt(tmpheight);
					int worh = r.nextInt(2);
					int wl, hl;
					if (tmp[nw][nh].getContentType() != Enum.WALL){
						tmp[nw][nh].updateContent(wall);
						i++;
						if (worh == 0){
							wl = r.nextInt(tmpwidth-2);
							if (wl > (walls-i)) wl = walls - i;
							for (int j=0; j<wl; j++){
								nw++;
								if (nw == tmpwidth) nw = 0;
								if (tmp[nw][nh].getContentType() != Enum.WALL){
									tmp[nw][nh].updateContent(wall);
									i++;
								}
							}
						}
						else{
							hl = r.nextInt(tmpheight-2);
							if (hl > (walls-i)) hl = walls-i;
							for (int j=0; j<hl; j++){
								nh++;
								if (nh == tmpheight) nh = 0;
								if (tmp[nw][nh].getContentType() != Enum.WALL){
									tmp[nw][nh].updateContent(wall);
									i++;
								}
							}
						}
					}
				}
			}
		}

		// Check if field is valid
		int[][] seen;
		seen = new int[fieldWidth][fieldHeight];

		// Shows the field in 2d map
		FieldCell[][] f;
		f = new FieldCell[fieldWidth][fieldHeight];
		for (int k=0, i=0; k<fieldHeight; k++){
			for (int j=0; j<fieldWidth; j++){
				f[j][k] = cells[i];
				if (f[j][k].getContentType() == Enum.WALL)
					seen[j][k] = 0;
				else seen[j][k] = -1;
				i++;
			}
		}
		for (int k=0, i=0; k<fieldHeight; k++){
			for (int j=0; j<fieldWidth; j++){
				if (seen[j][k] == -1){
					seen = traverseCell(seen, f, fieldWidth, fieldHeight, j, k);
					k = fieldHeight;
					j = fieldWidth;
				}
			}
		}

		for (int k=0, i=0; k<fieldHeight; k++){
			for (int j=0; j<fieldWidth; j++){
				if (seen[j][k] == -1){
					return false;
				}
			}
		}
	return true;
	}



/**
 * Helps the mapgenerator to determine if the map
 * generated is valid or not.
 *
 * @param seen An alternate field. If it contains a cell
 *             with value -1, the field is not valid.
 * @param f The field to check.
 * @param width Width of the field.
 * @param height Height of the field.
 * @param w The current position along the x axis.
 * @param h The current position along the y axis.
 */
	private int[][] traverseCell(int[][] seen, FieldCell[][] f, int width, int height, int w, int h){
		int htmp;
		int wtmp;
		// UP
		if (h == 0)
			htmp = height-1;
		else
			htmp = h-1;
		if (seen[w][htmp] == -1){
			if (f[w][htmp].getContentType() == Enum.WALL)
				seen[w][htmp] = 0;
			else if (f[w][htmp].getContentType() == Enum.EMPTY){
				seen[w][htmp] = 1;
				seen = traverseCell(seen, f, width, height, w, htmp);
			}
		}
		// RIGHT
		if (w == width-1)
			wtmp = 0;
		else
			wtmp = w+1;
		if (seen[wtmp][h] == -1){
			if (f[wtmp][h].getContentType() == Enum.WALL)
				seen[wtmp][h] = 0;
			else if (f[wtmp][h].getContentType() == Enum.EMPTY){
				seen[wtmp][h] = 1;
				seen = traverseCell(seen, f, width, height, wtmp, h);
			}
		}
		// DOWN
		if (h == height-1)
			htmp = 0;
		else
			htmp = h+1;
		if (seen[w][htmp] == -1){
			if (f[w][htmp].getContentType() == Enum.WALL)
				seen[w][htmp] = 0;
			else if (f[w][htmp].getContentType() == Enum.EMPTY){
				seen[w][htmp] = 1;
				seen = traverseCell(seen, f, width, height, w, htmp);
			}
		}
		// LEFT
		if (w == 0)
			wtmp = width-1;
		else
			wtmp = w-1;
		if (seen[wtmp][h] == -1){
			if (f[wtmp][h].getContentType() == Enum.WALL)
				seen[wtmp][h] = 0;
			else if (f[wtmp][h].getContentType() == Enum.EMPTY){
				seen[wtmp][h] = 1;
				seen = traverseCell(seen, f, width, height, wtmp, h);
			}
		}
		return seen;
	}



/**
 * Arranges cells into a 2d array
 */
	public FieldCell[][] get2d(){
		FieldCell[][] tmp = new FieldCell[fieldWidth][fieldHeight];
		for (int k=0, i=0; k<fieldHeight; k++){
			for (int j=0; j<fieldWidth; j++){
				tmp[j][k] = cells[i];
				i++;
			}
		}
		return tmp;
	}



/**
 * To create a given type of Berry
 *
 * @param type Type of berry
 * @param x Position on the field along the x axis
 * @param y Position on the field along the y axis
 */
	public void addBerry(String type, int x, int y){
		Random r = new Random();
		Wall wall = new Wall();
		Berry berry;
		if (type.equals("stone"))
			berry = new StoneBerry(this);
		else if (type.equals("saw"))
			berry = new SawBerry(this);
		else
			berry = new FieldBerry(this);

		int n = fieldWidth*y+x;

		if (x<0 && y<0){
			int tries = 0;
			while (tries < 10){
				n = r.nextInt(numCells);
				if (cells[n].getContentType() == Enum.EMPTY)
					tries = 10;
				else{
					tries++;
				}
			}
		}
		if (cells[n].getContentType() == Enum.EMPTY){
			cells[n].updateContent(berry);
		}
	}



/**
 * To create a Snake
 *
 * @param x Position on the field along the x axis
 * @param y Position on the field along the y axis
 * @param length Length of the snake
 * @param dir Position of the body compared to head
 */
	public boolean addSnakeToField(int x, int y, int length, int dir, Snake s){

		int n = fieldWidth*y+x;

		// Checks if the given position is valid.
		// Not valid: The cell is already occupied by something else.
		//            The cell is surrounded by at least 3 harming objects.
		if (x >= fieldWidth || y >= fieldHeight){
			//System.out.println("<< Error! Invalid position. Snake not created.");
			return false;
		}
		int wn = 0;
		for (int d=0; d<numNeighbours; d++){
			int type = (getNeighbour(n, d)).getContentType();
			if (type == Enum.WALL || type == Enum.SNAKEHEAD || type == Enum.SNAKEBODY)
				wn++;
		}
		if (wn >= 3){
			//System.out.println("<< Error! Snake surrounded. Can't find initial direction.");
			//System.out.println("<< Snake not created.");
			return false;
		}

		// Checks if the snake is too long
		if ((dir == Enum.LEFT || dir == Enum.RIGHT) && (length>=fieldWidth)){
			//System.out.println("<< Error! Snake is too long. Snake not created.");
			return false;
		}
		if ((dir == Enum.UP || dir == Enum.DOWN) && (length>=fieldHeight)){
			//System.out.println("<< Error! Snake is too long. Snake not created.");
			return false;
		}

		// Checks if the length is valid.
		if (length < 0){
			//System.out.println("<< Error! Invalid length. Snake not created.");
			return false;
		}

		// Checks if the x:y part of the field is empty
		if (cells[n].getContentType() != Enum.EMPTY){
			//System.out.println("<< Error! Field at "+ x +":" + y + " is not empty. Snake not created.");
			return false;
		}

		// Creats a 2d version of the field
		FieldCell[][] tmp = get2d();

		// Creates a new snake if none was given
		if (s == null){
			s = new Snake();
		}

		s.setField(this);

		// Only head is added with no body parts
		if (length == 0){
			if (s.getHead() != null){
				s.getHead().setPosition(tmp[x][y]);
				tmp[x][y].updateContent(s.getHead());
			}
			//System.out.println("<< Snake[" + s.getID() + "] has 1 head (at "+x+":"+y+") and 0 bodies, added to field.");
			initDirection(s.getHead());
			return true;
		}

		// Body parts are also added with head
		if (length > 0){
			FieldCell[] update = new FieldCell[length+1];
			update[0] = tmp[x][y];
			//System.out.println("<< Position of Snake[" + s.getID() + "]'s head: "+x+":"+y);
			int xtmp = x;
			int ytmp = y;
			// Checks if for the whole length of the snake all the cells of 
			// the field needed are empty
			for (int i=0; i<length; i++){
				switch(dir){
					case Enum.UP:    if (ytmp == 0) ytmp = fieldHeight-1;
									 else ytmp--;
									 break;
					case Enum.RIGHT: if (xtmp == fieldWidth-1) xtmp = 0;
									 else xtmp++;
									 break;
					case Enum.DOWN:  if (ytmp == fieldHeight-1) ytmp = 0;
									 else ytmp++;
									 break;
					case Enum.LEFT:  if (xtmp == 0) xtmp = fieldWidth-1;
									 else xtmp--;
									 break;
					default:		 if (xtmp == 0) xtmp = fieldWidth-1;
									 else xtmp--;
									 break;
				}
				if (tmp[xtmp][ytmp].getContentType() != Enum.EMPTY){
					//System.out.println("<< Error! There is something in the way of the snake at "+ xtmp +":" + ytmp + ". Snake not created.");
					return false;
				}
				//System.out.println("<< Position of bodypart["+i+"]: "+xtmp+":"+ytmp);
				update[i+1] = tmp[xtmp][ytmp];
			}
			// Other parts of the snake are created
			SnakePart[] parts = s.initLength(length, update);
			// Field is updated with the newly created parts of the snake
			if (parts != null){
				for (int i=0; i<parts.length; i++){
					update[i].updateContent(parts[i]);
				}
			}
			initDirection(s.getHead());
			// The new snake is added to the others if it did not exist before
			if (snakes == null){
				snakes = new Snake[1];
				snakes[0] = s;
			}
			else{
				for (int i=0; i<snakes.length; i++){
					if (snakes[i].getID() == s.getID()){
						//System.out.println("<< Snake[" + s.getID() + "] already added to others.");
						return true;
					}
				}
				Snake[] stmp = new Snake[snakes.length+1];
				for (int i=0; i<snakes.length; i++)
					stmp[i] = snakes[i];
				stmp[snakes.length] = s;
				snakes = stmp;
				//System.out.println("<< Snake[" + s.getID() + "] added to field.");
				return true;
			}
		}
		else{
			System.out.println("<< Invalid length.");
			//return false;
		}
		//System.out.println("<< Snake[" + s.getID() + "] added to field.");
		return true;
	}



/**
 * Adds snake created by biten off from others
 */
	public void addNewAISnake(Snake s){
		s.setField(this);
		if (snakes == null){
			snakes = new Snake[1];
			snakes[0] = s;
		}
		else{
			for (int i=0; i<snakes.length; i++){
				if (snakes[i].getID() == s.getID()){
					return;
				}
			}
			Snake[] stmp = new Snake[snakes.length+1];
			for (int i=0; i<snakes.length; i++)
				stmp[i] = snakes[i];
			stmp[snakes.length] = s;
			snakes = stmp;
			return;
		}
	}



/**
 * Initializes snake's direction after created
 */
	public void initDirection(SnakeHead head){
		Random r = new Random();
		FieldCell cell = head.getPosition();
		int c = getCellID(cell);
		head.initDirection(Enum.RIGHT);
		int dir = r.nextInt(numNeighbours);
		for (int d=0; d<numNeighbours && dir<numNeighbours; d++){
			dir++; dir =  dir%numNeighbours;
			int type = (cell.getNeighbour(dir)).getContentType();
			if (type != Enum.SNAKEHEAD && type != Enum.SNAKEBODY && type != Enum.WALL){
				head.initDirection(dir);
				dir = numNeighbours;
			}
		}
	}



/**
 * Updates berry's lifetime on the field
 */
	public void updateBerry(){
		for (int i=0; i<numCells; i++){
			int tmp = cells[i].getContentType();
			if (tmp == Enum.FIELDBERRY || tmp == Enum.SAWBERRY || tmp == Enum.STONEBERRY){
				((Berry)cells[i].getContent()).decreaseTimeBeforeDestroy();
			}
		}
	}



/**
 * Makes snake dead.
 *
 * @param id ID of snake to kill.
 */
	public void makeSnakeDead(int id){
		int n = 1;
		for (int i=0; i<numCells; i++){
			int tmp = cells[i].getContentType();
			if (tmp == Enum.SNAKEBODY || tmp == Enum.SNAKEHEAD){
				int type;
				if ((((SnakePart)(cells[i].getContent())).snake).getID() == id){
					if (tmp == Enum.SNAKEHEAD) type = Enum.DEADSNAKEHEAD;
					else type = Enum.DEADSNAKEBODY;
					boolean stone = ((SnakePart)(cells[i].getContent())).getStoneStatus();
					int ownDir = ((SnakePart)(cells[i].getContent())).getDirection();
					int nextDir = -1;
					if (((SnakePart)(cells[i].getContent())).hasNext())
						nextDir = (((SnakePart)(cells[i].getContent())).getNext()).getDirection();
					SnakeDead sd = new SnakeDead(type, cells[i], stone, ownDir, nextDir, n);
					cells[i].updateContent(sd);
					n++;
				}
			}
		}
	}



/**
 * Makes dead snakes decay.
 */
	public void makeSnakeDecay(){
		for (int i=0; i<numCells; i++){
			int tmp = cells[i].getContentType();
			if (tmp == Enum.DEADSNAKEHEAD || tmp == Enum.DEADSNAKEBODY){
				((SnakeDead)(cells[i].getContent())).decay();
			}
		}
	}



/**
 * Removes Berry from the field
 *
 * @param id ID of berry to remove from the field
 */
	public void removeBerryContent(int id){
		Empty empty = new Empty();
		for (int i=0; i<numCells; i++){
			if ((cells[i].getContentType() == Enum.FIELDBERRY) || (cells[i].getContentType() == Enum.SAWBERRY) || (cells[i].getContentType() == Enum.STONEBERRY)){
				if ((cells[i].getContent()).getID() == id){
					cells[i].updateContent(empty);
					return;
				}
			}
		}
	}

}
