/**
 * Class FieldCell
 * Cells of the Field, each containing an IContent.
 * It knows about its neigbours, it tells its
 * content which snake it was hit by.
 */
public class FieldCell {

    // The cell's content.
	private IContent content;

    // The cell's neighbours.
	private FieldCell[] neighbours;

	// Reference to the Field object.
	private Field field;

	// Number of FieldCell objects
	public static int idnum = 0;

	// ID of an exact FieldCell object
	private int id;



/**
 * Class constructor.
 */
	public FieldCell(int numNeighbours) {
		id = idnum; idnum++;
		neighbours = new FieldCell[numNeighbours];
    }



/**
 * Sets the neighbour of this cell in the given direction.
 *
 * @param direction The direction in which the neighbour is set.
 * @param neighbouringCell The neighbour that will be in the given direction.
 */   
 
	public void setNeighbour(int direction, FieldCell neighbouringCell){
		neighbours[direction] = neighbouringCell;
    }



 /**
 * Gets the neighbour of this cell int the given diretion
 */   
 
	public FieldCell getNeighbour(int direction){
		return neighbours[direction];
    }



/**
 * Updates the content of this cell with the given object.
 *
 * @param setcontent An exact IContent that will be on this FieldCell.
 */ 
	public void updateContent(IContent setcontent){
		content = setcontent;
    }



/**
 * Returns the content.
 */ 
	public IContent getContent(){
		return content;
    }



/**
 * Calls the content's reaction to the collision and forwards
 * the Snake object reference.
 *
 * @param from The Snake's reference that collided with the content of the FieldCell.
 */
	public void collide(Snake from) {    
		content.reactToCollision(from);

    }



/**
 * Returns the type if IContent
 */
	public int getContentType(){
		if (content != null) return content.getType();
		else return Enum.UNKNOWN;
	}



}
