import java.util.Random;
import java.awt.event.KeyEvent;

/**
 * Class Game
 * Handles the game itself, makes time pass.
 * Creates newgames, and determines the winner.
 */
public class Game {

 	// The Field on which the game is played.
	public Field gameField;

	// Players controlled by the computer
	private AIPlayer aiplayer[];

	// Players controlled by human players.
	private HumanPlayer huplayer[];

	// Tick's length
	private int tick;

	// Game over
	private static boolean gameover = false;
	public Snake[] winner;

	// Berries's appearing time
	private int timeTillFieldBerry;
	private int timeTillSawBerry;
	private int timeTillStoneBerry;

	// Draw field in every 5 ticks
	private int draw = 0;

	// Renderer
	private Renderer renderer;

/**
 * Class constructor.
 */
	public Game(){
	}

/**
 * Checks if game end
 */
	public boolean gameOver(){
		return gameover;
	}

/**
 * Reloads the time
 */
	public void resetAllTimeValues(){
		tick = Enum.TICKTIME;
		resetBerryTime(Enum.FIELDBERRY);
		resetBerryTime(Enum.SAWBERRY);
		resetBerryTime(Enum.STONEBERRY);
	}

/**
 * Starts a new game.
 */    
	public void startNewGame(int width, int height, boolean wraparound, float wallpercentage, String method, int aiplayers, int huplayers, RenderTarget rt){
		
		gameover = false;
		resetAllTimeValues();

		Controller.idnum = 0;
		aiplayer = null;
		huplayer = null;

		gameField = null;

		if (huplayers < 0){
			//System.out.println("<< Warning! Number of human players is smaller than 0. Set to 0.");
			huplayers = 0;
		}
		else{
			huplayer = new HumanPlayer[huplayers];
			for (int i=0; i<huplayers; i++){
				huplayer[i] = new HumanPlayer();

			}
		}

		if (aiplayers < 0){
			//System.out.println("<< Warning! Number of AI players is smaller than 0. Set to 0.");
			aiplayers = 0;
		}
		else{
			aiplayer = new AIPlayer[aiplayers];
			for (int i=0; i<aiplayers; i++)
				aiplayer[i] = new AIPlayer();
		}

		renderer = new Renderer(rt);
		gameField = new Field(this);
		gameField.generateField(width, height,wraparound, wallpercentage, method, aiplayers+huplayers, aiplayer, huplayer, renderer);
	}



/**
 * Computer takes the control over the cut part of a snake
 */
	 public void giveAI(Snake s){
		if (aiplayer == null){
			aiplayer = new AIPlayer[1];
			aiplayer[0] = new AIPlayer();
			aiplayer[0].setField(gameField.get2d());
			aiplayer[0].setSnake(s);
		}
		else{
			AIPlayer aip[];
			aip = new AIPlayer[aiplayer.length+1];
			for (int i=0; i<aiplayer.length; i++)
				aip[i] = aiplayer[i];
			aip[aiplayer.length] = new AIPlayer();
			aip[aiplayer.length].setField(gameField.get2d());
			aip[aiplayer.length].setSnake(s);
			aiplayer = aip;
		}
		gameField.addNewAISnake(s);
	}



/**
 * Determines which player won the game.
 */
	public Snake[] determineWinner(){
		Snake s[] = new Snake[1];
		int maxLength = -1;
		int num = 0;
		// Computer player
		if (aiplayer != null){
			for (int i=0; i<aiplayer.length; i++){
				int l = (aiplayer[i].getSnake()).getLength();
				// When new length is bigger
				if (l > maxLength){
					num = 1;
					maxLength = l;
					s = new Snake[num];
					s[num-1] = aiplayer[i].getSnake();
				}
				// When length is the same
				else if (l == maxLength){
					num++;
					Snake stmp[] = new Snake[num];
					for (int j=0; j<num-1; j++){
						stmp[j]=s[j];
					}
					stmp[num-1] = aiplayer[i].getSnake();
					s = stmp;
				}
			}
		}
		// Human players
		if (huplayer != null){
			for (int i=0; i<huplayer.length; i++){
				int l = (huplayer[i].getSnake()).getLength();
				// When new length is bigger
				if (l > maxLength){
					num = 1;
					maxLength = l;
					s = new Snake[num];
					s[num-1] = huplayer[i].getSnake();
				}
				// When length is the same
				else if (l == maxLength){
					num++;
					Snake stmp[] = new Snake[num];
					for (int j=0; j<num-1; j++){
						stmp[j]=s[j];
					}
					stmp[num-1] = huplayer[i].getSnake();
					s = stmp;
				}
			}
		}
		if (num == 0 || maxLength == 0)
			return s;

		int minStones = -1;
		Snake s2[] = new Snake[1];
		num = 0;
		for (int i=0; i<s.length; i++){
			int st = s[i].getNumOfStones();
			// When number of stones is smaller
			if (minStones == -1 || st < minStones){
				num = 1;
				minStones = st;
				s2 = new Snake[num];
				s2[num-1] = s[i];
			}
			// When number of stones is the same
			else if (st == minStones){
				num++;
				Snake stmp[] = new Snake[num];
				for (int j=0; j<num-1; j++){
					stmp[j]=s2[j];
				}
				stmp[num-1] = s[i];
				s2 = stmp;
			}
		}
		return s2;
	}

/**
 * Check the end of the game
 */
	public boolean checkEndOfGame(){
		int snakes = 0;
		int deadsnakes = 0;
		if (aiplayer != null) snakes += aiplayer.length;
		if (huplayer != null) snakes += huplayer.length;
		if (snakes == 0){
			//System.out.println("<< There are no snakes.");
			return false;
		}
		else{
			if (aiplayer != null){
				for (int i=0; i<aiplayer.length; i++){
					if (!((aiplayer[i].getSnake()).alive))
						deadsnakes++;
				}
			}
			if (huplayer != null){
				for (int i=0; i<huplayer.length; i++){
					if (!((huplayer[i].getSnake()).alive))
						deadsnakes++;
				}
			}
		}
		if (deadsnakes == snakes){
			Snake s[] = determineWinner();
			if (s == null)
				return false;
			winner = s;
			System.out.println("<< Winner snake(s): " + s.length);
			for (int i=0; i<s.length; i++){
				System.out.println("<<\t Congratulation to Snake[" + s[i].getID() + "]!");
				s[i].getStatus();
			}
			gameover = true;
			return true;
		}
		return false;
	}



/**
 * Resets time till the next Berry appears.
 */
	public void resetBerryTime(int type){
		Random r = new Random();
		switch(type){
			case Enum.FIELDBERRY:
				timeTillFieldBerry = (r.nextInt(Enum.TRANDOMFIELD)) + Enum.TMINFIELD;
				break;
			case Enum.SAWBERRY:
				timeTillSawBerry   = (r.nextInt(Enum.TRANDOMSAW))   + Enum.TMINSAW;
				break;
			case Enum.STONEBERRY:
				timeTillStoneBerry = (r.nextInt(Enum.TRANDOMSTONE)) + Enum.TMINSTONE;
				break;
		}
	}



/**
 * Ticks till the game is over
 */
	public void tickTillEnd(){
		if (!gameover){
			tick();
			
		}
		//else System.out.println("<< The game ended.");
	}


	public int getNumberOfPlayers(){
		int n=0;
		if (aiplayer != null)
			n += aiplayer.length;
		if (huplayer != null)
			n += huplayer.length;

		return n;
	}


/**
 * Ticks when a given amount of time elapses.
 */
	public void tick(){
		gameField.makeSnakeDecay();

		// Updates berry's time before destorying
		gameField.updateBerry();	

		// Berry's time decreased
		timeTillFieldBerry--;
		timeTillSawBerry--;
		timeTillStoneBerry--;

		// Tries to create a Berry if neeeded
		if (timeTillFieldBerry < 0){
			resetBerryTime(Enum.FIELDBERRY);
			gameField.addBerry("field", -1, -1);
		}
		if (timeTillSawBerry < 0){
			resetBerryTime(Enum.SAWBERRY);
			gameField.addBerry("saw", -1, -1);
		}
		if (timeTillStoneBerry < 0){
			resetBerryTime(Enum.STONEBERRY);
			gameField.addBerry("stone", -1, -1);
		}
		if (!gameover && !checkEndOfGame()){
			if (aiplayer != null){
				for (int i=0; i<aiplayer.length; i++){
					if (aiplayer[i] != null)
						aiplayer[i].decideNextMove();
				}
			}
			gameField.moveSnakes();
		}
		//else System.out.println("<< The game ended.");
		renderer.renderScene();
//		gameField.drawField();
	}

/**
 * Handles the key typed
 */
	public void keyTyped(KeyEvent e) {
		int location = e.getKeyCode();
		switch(location){
			case Enum.EXIT:
				System.exit(0);
				break;
			// Speeds up gamespeed
			case Enum.SPEEDUP:
				if (Enum.SNAKEMOVETIME>1){
					Enum.SNAKEMOVETIME = Enum.SNAKEMOVETIME-1;
					System.out.println("Snakes move faster (in every "+Enum.SNAKEMOVETIME+" tick(s)).");
				}
				break;
			// Slowers gamespeed
			case Enum.SPEEDDOWN:
				Enum.SNAKEMOVETIME = Enum.SNAKEMOVETIME+1;
				System.out.println("Snakes move slower (in every "+Enum.SNAKEMOVETIME+" tick(s)).");
				break;
			// Speeds up stonespeed
			case Enum.STONEUP:
				if (Enum.STONEROLLTIME>1){
					Enum.STONEROLLTIME = Enum.STONEROLLTIME-1;
					System.out.println("Stones move faster (in every "+Enum.STONEROLLTIME+" tick(s)).");
				}
				break;
			// Slowers stonespeed
			case Enum.STONEDOWN:
				Enum.STONEROLLTIME = Enum.STONEROLLTIME+1;
				System.out.println("Stones move slower (in every "+Enum.STONEROLLTIME+" tick(s)).");
				break;
			// Player1
			case Enum.PL1_UP:
				if (huplayer != null && huplayer.length > Enum.PLAYER1){
					huplayer[Enum.PLAYER1].giveDirection(Enum.UP);
				}
				break;
			case Enum.PL1_RIGHT:
				if (huplayer != null && huplayer.length > Enum.PLAYER1){
					huplayer[Enum.PLAYER1].giveDirection(Enum.RIGHT);
				}
				break;
			case Enum.PL1_DOWN:
				if (huplayer != null && huplayer.length > Enum.PLAYER1){
					huplayer[Enum.PLAYER1].giveDirection(Enum.DOWN);
				}
				break;
			case Enum.PL1_LEFT:
				if (huplayer != null && huplayer.length > Enum.PLAYER1){
					huplayer[Enum.PLAYER1].giveDirection(Enum.LEFT);
				}
				break;
			// Player 2
			case Enum.PL2_UP:
				if (huplayer != null && huplayer.length > Enum.PLAYER2)
					huplayer[Enum.PLAYER2].giveDirection(Enum.UP);
				break;
			case Enum.PL2_RIGHT:
				if (huplayer != null && huplayer.length > Enum.PLAYER2)
					huplayer[Enum.PLAYER2].giveDirection(Enum.RIGHT);
				break;
			case Enum.PL2_DOWN:
				if (huplayer != null && huplayer.length > Enum.PLAYER2)
					huplayer[Enum.PLAYER2].giveDirection(Enum.DOWN);
				break;
			case Enum.PL2_LEFT:
				if (huplayer != null && huplayer.length > Enum.PLAYER2)
					huplayer[Enum.PLAYER2].giveDirection(Enum.LEFT);
				break;
			// Player 3
			case Enum.PL3_UP:
				if (huplayer != null && huplayer.length > Enum.PLAYER3)
					huplayer[Enum.PLAYER3].giveDirection(Enum.UP);
				break;
			case Enum.PL3_RIGHT:
				if (huplayer != null && huplayer.length > Enum.PLAYER3)
					huplayer[Enum.PLAYER3].giveDirection(Enum.RIGHT);
				break;
			case Enum.PL3_DOWN:
				if (huplayer != null && huplayer.length > Enum.PLAYER3)
					huplayer[Enum.PLAYER3].giveDirection(Enum.DOWN);
				break;
			case Enum.PL3_LEFT:
				if (huplayer != null && huplayer.length > Enum.PLAYER3)
					huplayer[Enum.PLAYER3].giveDirection(Enum.LEFT);
				break;
			// Player 4
			case Enum.PL4_UP:
				if (huplayer != null && huplayer.length > Enum.PLAYER4)
					huplayer[Enum.PLAYER4].giveDirection(Enum.UP);
				break;
			case Enum.PL4_RIGHT:
				if (huplayer != null && huplayer.length > Enum.PLAYER4)
					huplayer[Enum.PLAYER4].giveDirection(Enum.RIGHT);
				break;
			case Enum.PL4_DOWN:
				if (huplayer != null && huplayer.length > Enum.PLAYER4)
					huplayer[Enum.PLAYER4].giveDirection(Enum.DOWN);
				break;
			case Enum.PL4_LEFT:
				if (huplayer != null && huplayer.length > Enum.PLAYER4)
					huplayer[Enum.PLAYER4].giveDirection(Enum.LEFT);
				break;

		}
	}


}

