import java.awt.BorderLayout;
import java.awt.Dimension;

import java.awt.FlowLayout;
import java.awt.Rectangle;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.Timer;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.BorderFactory;
import javax.swing.JTextArea;
import javax.swing.JOptionPane;

/**
 * Class JFrame
 * Tha Game's main class,
 */
public class MainWindow extends JFrame {

	//the object derived from JPanel on which the scene is drawn
	private RenderTarget renderTarget = new RenderTarget();
	
	//the layout of the JFrame
	private BorderLayout layout = new BorderLayout();

	//the layout of the subpanel under the renderTarget object
	private FlowLayout sublayout = new FlowLayout();

	//the subpanel under the renderTarget object
	private JPanel subpanel = new JPanel();

	//the subpanel when winning
	private JPanel winpanel = new JPanel();

	//the "New Game" button
	private JButton btnNewGame = new JButton();

	//the "Pause" button
	private JButton btnPause = new JButton();

	//the "Exit" button
	private JButton btnExit = new JButton();

	// The game itself
	private static Game game;

	// Timer of the game
	private static Timer t;


	KeyListener listener = new KeyListener(){
		public void keyPressed(KeyEvent e){
			int location = e.getKeyCode();
			if (location == Enum.PAUSE){
				if (t.isRunning())
					t.stop();
				else
					t.start();
			}
			else
				game.keyTyped(e);
		}
		public void keyReleased(KeyEvent e){
		}
		public void keyTyped(KeyEvent e){
		}
	};

	
	public void setNewGameValues(int width, int height, boolean wraparound, float walls, String method, int ai, int human) {
		

		game = new Game();
		game.startNewGame(width, height, wraparound, walls, method, ai, human, renderTarget);

		this.addKeyListener(listener);
	
		int delay = Enum.TICKTIME;
		ActionListener taskPerformer = new ActionListener(){
			public void actionPerformed(ActionEvent evt){
				if (!game.gameOver())
					game.tick();
				else{
					destroyMe(t);
					winner();
				}
			}
		};

		btnNewGame.setEnabled(false);
		btnPause.setEnabled(true);

		t = new Timer(delay, taskPerformer);
		t.setRepeats(true);
		t.start();
	}



	public MainWindow() {
		try {
			jbInit();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}



	public void winner(){

		Snake[] wsnake = game.winner;
		String wt = "The winner(s) of Snake Farm ("+wsnake.length+"/"+game.getNumberOfPlayers()+"):\n";
		if (wsnake != null){
			for (int i=0; i<wsnake.length; i++){
				wt = wt + ". Snake["+wsnake[i].getID()+"]:\n";
				wt = wt + "    controlled by ";
				switch (wsnake[i].getController()){
					case Enum.PLAYER1:  wt = wt + "Player1\n";  break;
					case Enum.PLAYER2:  wt = wt + "Player2\n";  break;
					case Enum.PLAYER3:  wt = wt + "Player3\n";  break;
					case Enum.PLAYER4:  wt = wt + "Player4\n";  break;
					case Enum.PLAYERAI: wt = wt + "Computer\n";  break;
					default:            wt = wt + "Unknown\n";  break;
				}
				wt = wt + "    length " + (wsnake[i].getLength()+1) + " with ";
				wt = wt + wsnake[i].getNumOfStones() + " stone(s)\n";
			}
		}
		System.out.println(wt);

		this.repaint();
		JOptionPane.showMessageDialog(null, wt, "Congratulations!", JOptionPane.PLAIN_MESSAGE);
	}



	public void destroyMe(Timer t) {
		try {
			t.stop();
		} catch (Exception e) {
		}
		t = null;
		btnNewGame.setEnabled(true);
		btnPause.setEnabled(false);
	}



	private void jbInit() throws Exception {
		this.getContentPane().setLayout(layout);
		this.setSize( new Dimension(506, 571) );
		this.setTitle("SnakeFarm by Team Zirnitra");
		this.setResizable(false);
		this.setFocusable(true);

		renderTarget.setSize(500, 500);
		this.getContentPane().add(renderTarget, layout.CENTER);
 
		subpanel.setSize(506, 40);
		subpanel.setLayout(sublayout);
		this.getContentPane().add(subpanel, layout.SOUTH);

		btnNewGame.setText("New Game");
		btnNewGame.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				btnNewGame_actionPerformed(e);
			}
		});
		btnPause.setText("Pause");
		btnPause.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				btnPause_actionPerformed(e);
			}
		});
		btnExit.setText("Exit");
		btnExit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				btnExit_actionPerformed(e);
			}
		});

		subpanel.add(btnNewGame, layout.SOUTH);
		subpanel.add(btnPause, layout.SOUTH);
		btnPause.setEnabled(false);
		subpanel.add(btnExit, layout.SOUTH);
    }



	private void btnExit_actionPerformed(ActionEvent e) {
		this.dispose();
		System.exit(1);
    }



	private void btnNewGame_actionPerformed(ActionEvent e) {
		SettingsPanel stpanel = new SettingsPanel(this);
		stpanel.setVisible(true);
    }



	private void btnPause_actionPerformed(ActionEvent e) {
		if (t.isRunning())
			t.stop();
		else {
			t.start();
			this.requestFocusInWindow();
		}
    }



}
