

import java.awt.Color;
import java.awt.Dimension;

import java.awt.FlowLayout;

import java.awt.Rectangle;

import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JToggleButton;

import javax.swing.SpinnerNumberModel;

public class SettingsPanel extends JFrame {

	private JLabel lblHumanPlayers = new JLabel();
	private JSpinner spnHumanPlayers = new JSpinner();
	private JLabel lblAIPlayers = new JLabel();
	private JSpinner spnAIPlayers = new JSpinner();
	private JPanel pnlPlayers = new JPanel();
	private JPanel pnlMap = new JPanel();
	private JLabel lblWidth = new JLabel();
	private JLabel lblHeight = new JLabel();
	private JSpinner spnWidth = new JSpinner();
	private JSpinner spnHeight = new JSpinner();
	private JCheckBox chbWraparound = new JCheckBox();
	private JLabel lblWallPercentage = new JLabel();
	private JSlider jSlider1 = new JSlider();
	private JLabel lblMethod = new JLabel();
	private JButton btnStart = new JButton();
	private JButton btnCancel = new JButton();
	private final String[] methods = {"Room-like", "Random"};
	private JComboBox cmbMethod = new JComboBox(methods);
	
	private MainWindow mw;



	public SettingsPanel(MainWindow mw) {

		this.mw = mw;
		
		try {
			jbInit();
		} catch (Exception e) {
			e.printStackTrace();
		}
	

	}



	private void jbInit() throws Exception {

		this.getContentPane().setLayout(null);
		this.setSize(new Dimension(Enum.G_MENUX, Enum.G_MENUY));
		this.setTitle("Settings");
		this.setResizable(false);

		lblHumanPlayers.setText("HU players :");
		spnHumanPlayers.setSize(new Dimension(35, 21));
		spnHumanPlayers.setPreferredSize(new Dimension(35, 21));
		spnHumanPlayers.setToolTipText("You can set the number of human players in the game here.");
		spnHumanPlayers.setModel(new SpinnerNumberModel(1, Enum.HUPLAYERMIN, Enum.HUPLAYERMAX, 1));
		lblAIPlayers.setText("AI players :");
		spnAIPlayers.setPreferredSize(new Dimension(35, 21));
		spnAIPlayers.setSize(new Dimension(35, 21));
		spnAIPlayers.setToolTipText("You can set the number of AI players in the game here.");
		spnAIPlayers.setModel(new SpinnerNumberModel(1, Enum.AIPLAYERMIN, Enum.AIPLAYERMAX, 1));
		pnlPlayers.setBounds(new Rectangle(15, 20, 150, 100));
		pnlPlayers.setBorder(BorderFactory.createTitledBorder("Players"));
		pnlPlayers.setFocusable(false);
		pnlMap.setBounds(new Rectangle(15, 135, 150, 205));
		pnlMap.setBorder(BorderFactory.createTitledBorder("Map"));
		lblWidth.setText("Width :");
		lblHeight.setText("Height :");
		spnWidth.setPreferredSize(new Dimension(50, 21));
		spnWidth.setToolTipText("You can set the playfield's width (in map squares).");
		spnWidth.setModel(new SpinnerNumberModel(Enum.WIDTHMIN, Enum.WIDTHMIN, Enum.WIDTHMAX, 1));
		spnHeight.setPreferredSize(new Dimension(50, 21));
		spnHeight.setToolTipText("You can set the playfield's height (in map squares).");
		spnHeight.setModel(new SpinnerNumberModel(Enum.HEIGHTMIN, Enum.HEIGHTMIN, Enum.HEIGHTMAX, 1));
		chbWraparound.setText("Wraparound");
		chbWraparound.setToolTipText("You can enable or disable wraparound (snakes that quit the playfield reenter on the opposite side).");
		lblWallPercentage.setText("Wall percentage :");
		jSlider1.setPreferredSize(new Dimension(90, 16));
		jSlider1.setMaximum(10);
		jSlider1.setValue(5);
		jSlider1.setToolTipText("You can set which percentage of the playfield's squares are occupied by walls.");
		lblMethod.setText("Method :");
		btnStart.setText("Start");
		btnStart.setBounds(new Rectangle(15, 345, 65, 25));
		btnStart.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				btnStart_actionPerformed(e);
			}
		});
		btnCancel.setText("Cancel");
		btnCancel.setBounds(new Rectangle(85, 345, 75, 25));
		btnCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				btnCancel_actionPerformed(e);
			}
		});
		cmbMethod.setToolTipText("You can set which method is used to generate the playfield. Room-like gives something lika a labyrinth, while random places walls in a random fashion.");
		cmbMethod.setPreferredSize(new Dimension(90, 21));
		pnlPlayers.add(lblHumanPlayers, null);
		pnlPlayers.add(spnHumanPlayers, null);
		pnlPlayers.add(lblAIPlayers, null);
		pnlPlayers.add(spnAIPlayers, null);
		pnlMap.add(lblWidth, null);
		pnlMap.add(spnWidth, null);
		pnlMap.add(lblHeight, null);
		pnlMap.add(spnHeight, null);
		pnlMap.add(chbWraparound, null);
		pnlMap.add(lblWallPercentage, null);
		pnlMap.add(jSlider1, null);
		pnlMap.add(lblMethod, null);
		pnlMap.add(cmbMethod, null);

		this.getContentPane().add(btnCancel, null);
		this.getContentPane().add(btnStart, null);
		this.getContentPane().add(pnlMap, null);
		this.getContentPane().add(pnlPlayers, null);

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		this.setLocation( ( screenSize.width - this.getSize().width ) / 2, ( screenSize.height - this.getSize().height ) / 2 );

	}



	private void btnCancel_actionPerformed(ActionEvent e) {

		this.dispose();

	}
	
	
	private void btnStart_actionPerformed(ActionEvent e) {
		String method;
		if (cmbMethod.getSelectedIndex() == 0) method = "room";
		else method = "random";
		mw.setNewGameValues(((Integer)spnWidth.getValue()).intValue(), ((Integer)spnHeight.getValue()).intValue(), chbWraparound.isSelected(), jSlider1.getValue(), method, ((Integer)spnAIPlayers.getValue()).intValue(), ((Integer)spnHumanPlayers.getValue()).intValue());
		this.dispose();

	}

}
