/**
 * Class Snake
 * Represents snakes as a whole entity.
 * Handles actions performed by and on the snakes.
 */
public class Snake {

	//reference to the snake's head element
	private SnakeHead head;

	//the snake's current state
	private int state;

	//remaining time, until berserked state is lost
	private int berserkTimeRemaining = 0;

	//reference to the Field object
	public Field field;

	// number of snake objects
	public static int idnum = 0;

	// id of an exact snake object
	public int id;

	// time till move
	public int timeTillMove;

	// life status
	public boolean alive;

	// Grows
	private boolean grow;

	// Controller
	private int controller = Enum.NOCONTROLLER;



/**
 * Class constructor.
 */
    public Snake() {
		id = idnum; idnum++;
		head = new SnakeHead(this);
		enableState(SnakeState.STATE_NORMAL);
		timeTillMove = Enum.SNAKEMOVETIME;
		alive = true;
		grow = false;
    }



	public SnakeHead getHead(){
		return head;
	}



	public void setHead(SnakeHead h){
		head = h;
	}



/**
 * Sets the field.
 */
	public void setField(Field f){
		field = f;
	}



/**
 * Returns the field.
 */
	public Field getField(){
		return field;
	}



/**
 * Sets the controller.
 */
	public void setController(int c){
		controller = c;
	}



/**
 * Returns the controller.
 */
	public int getController(){
		return controller;
	}


/**
 * Returns remaining time of berserk status
 */
	public int getBerserkTime(){
		return	berserkTimeRemaining;

	}

/**
 * The snake's body grows by one unit.
 */ 
    private void grow() {
		grow = true;
    }



/**
 * Returns the state of the snake.
 */ 
    public int getState() {
		return(state);
    }



/**
 * A part of the snake's body is biten off.
 *
 * @param from Part of a snake to be cut off from.
 */ 
    public void bitenOff(SnakePart from){
		// If head is hit by a berserked snake it dies
		if (from.getType() == Enum.SNAKEHEAD){
			from.snake.die();
			return;
		}
		// This bodypart is detached from the original snake
		(from.getPrev()).setNext(null);
		// From this part all body parts are disconnected from snake
		from.setPrev(null);
		// This part is deleted from the Field.
		from.getPosition().updateContent(new Empty());

		// If there are more body parts AI takes control over it 
		if (from.getNext() != null){

			int length = 0;
			SnakePart[] old = new SnakePart[1];

			// Creates a list of the bodyparts cutten
			SnakePart sb = from;
			sb = sb.getNext();
			while (sb != null){
				length++;
				if (length == 1)
					old[0] = sb;
				else{ 
					SnakePart[] tmp = new SnakePart[length];
					for (int i=0; i<length-1; i++){
						tmp[i] = old[i];
					}
					tmp[length-1] = sb;
					old = tmp;
				}
				sb = sb.getNext();
			}
			// Creates a new snake and gives head at the last
			// position of the part biten off
			Snake s = new Snake();
			s.disableState(SnakeState.STATE_BERSERKED);
			SnakeHead h = new SnakeHead(s);
			h.setPosition(old[old.length-1].getPosition());
			(h.getPosition()).updateContent(h);
			s.setHead(h);
			old[old.length-1] = h;
			// Other parts are added
			if (old.length == 1)
				h.setNext(null);
			else{
				// Next elements set
				SnakePart tmp = h;
				for (int i=old.length-1; i>0; i--){
					tmp = old[i];
					tmp.setNext((SnakeBody)old[i-1]);
					tmp = tmp.getNext();
					tmp.setSnake(s);
				}
				tmp.setNext(null);
				// Previous elements set
				for (int i=0; i<old.length-1; i++){
					tmp = old[i];
					tmp.setPrev(old[i+1]);
					tmp = tmp.getPrev();
				}
			}
			// The snake's initial direction is set
			field.initDirection(s.head);
			// Snake gets a computer controller
			field.game.giveAI(s);
			//s.getStatus();
		}
    }



/**
 * The snake moves to the next cell on the field.
 */ 
    public void move() {
		if (alive){
			decrementBerserkTime();
			head.rollStone();
			if (timeTillMove > 0)
				timeTillMove--;
			if (timeTillMove == 0){
				timeTillMove = Enum.SNAKEMOVETIME;
				if (head != null){
					head.collide();
					if (alive){
						head.move(grow);
						grow = false;
					}
				}
			}
		}
    }



/**
 * The snake dies.
 */ 
    public void die() {
		alive = false;
		field.makeSnakeDead(id);
    }



/**
 * Decreases the time remaining until berserked state is lost.
 */ 
    public void decrementBerserkTime(){
		if (berserkTimeRemaining > 0){
			berserkTimeRemaining--;
			if (berserkTimeRemaining == 0)
				disableState(SnakeState.STATE_BERSERKED);
		}
    }



/**
 * Resets the berserked state's timer.
 */ 
    public void resetBerserkTime() {
		enableState(SnakeState.STATE_BERSERKED);
		berserkTimeRemaining = Enum.BERSERKEDTIME;
    }



/**
 * Checks if has FULLOFSTONES status or not.
 */ 
    public void checkIfFullOfStones(){
		if (SnakeState.isFullOfStones(state)){
			return;
		}
		int stones = 0;
		int length = getLength()+1;
		if (head != null){
			for (int i=0; i<length; i++){
				if (head.hasStone(i)) stones++;
			}
			if (stones == length){
				enableState(SnakeState.STATE_FULLOFSTONES);
			}
		}
    }



/**
 * The snake collides with an empty cell, callback function.
 */ 
    public void collideWithEmpty() {
    }



/**
 * The snake collides with a cell containing a died (stoned) snakepart,
 * callback function.
 *
*/
    public void collideWithStonedDeadBody() {
		die();
    }



/**
 * The snake collides with a cell containing a died (not stoned) snakepart,
 * callback function.
 */ 
    public void collideWithDeadBody() {
	if (!SnakeState.isBerserked(state))
		die();
    }



/**
 * The snake collides with a cell containing another (normal) snake,
 * callback function.
 *
 * @param to The snake hit by this snake.
*/ 
    public void collideWithNormalSnake(Snake to) {
		if (!SnakeState.isBerserked(state))
			die();
    }



/**
 * The snake collides with a cell containing another (stoned) snake,
 * callback function.
 *
 * @param to The snake hit by this snake.
*/
    public void collideWithStonedSnake(Snake to) {
		die();
    }



/**
 * The snake collides with a cell containing another (berserked) snake,
 * callback function.
 *
 * @param to The snake hit by this snake.
 */
	public void collideWithBerserkedSnake(Snake to) {
		if (!SnakeState.isBerserked(state))
			die();
	}



/**
 * The snake collides with a wall cell, callback function.
 */   
    public void collideWithWall() {
        die();
    }



/**
 * The snake collides with a cell containig a FieldBerry, callback function.
 */ 
    public void collideWithFieldBerry() {
        if (!SnakeState.isFullOfStones(state)){
        	grow();
		}
    }



/**
 * The snake collides with a cell containig a SawBerry, callback function.
 */ 
    public void collideWithSawBerry() {
        if (!SnakeState.isFullOfStones(state)){
			enableState(SnakeState.STATE_BERSERKED);
			resetBerserkTime();
		}
    }



/**
 * The snake collides with a cell containig a StoneBerry, callback function.
 */ 
    public void collideWithStoneBerry() {
        if (!SnakeState.isFullOfStones(state)){
			head.receiveStone();
			checkIfFullOfStones();
		}
        else
			die();
    }



/**
 * Enables a state (NORMAL/BERSERKED/FULLOFSTONE).
 *
 * @param enablestate State to enable.
 */ 
    public void enableState(int enablestate) {
		if (SnakeState.isNormal(enablestate))
			state = SnakeState.STATE_NORMAL;
		else if (SnakeState.isFullOfStones(enablestate))
			 state |= SnakeState.STATE_FULLOFSTONES;
		else if (SnakeState.isBerserked(enablestate))
			state |= SnakeState.STATE_BERSERKED;
    }



/**
 * Disables a state (BERSERKED/FULLOFSTONE).
 *
 * @param disablestate State to disable.
 */
    public void disableState(int disablestate) {
		if (SnakeState.isFullOfStones(disablestate))
			 state &= ~(SnakeState.STATE_FULLOFSTONES);
		else if (SnakeState.isBerserked(disablestate))
			state &= ~(SnakeState.STATE_BERSERKED);
    }



/**
 * Gives the direction given by the controller to the head.
 *
 * @param dir Direction determined by the controller.
 */
	public void setDirection(int dir){
		head.setDirection(dir);
	}



/**
 * Returns the direction of the head.
 */
	public int getDirection(){
		return head.getDirection();
	}



/**
 * Gets length
 */
	public int getLength(){
		if (head != null)
			return head.getLength();
		else
			return 0;
	}



/**
 * Gets number of stones
 */
	public int getNumOfStones(){
		if (head != null)
			return head.getNumOfStones();
		else
			return 0;
	}



/**
 * Sets length
 */
	public SnakePart[] initLength(int length, FieldCell[] pos){
		SnakePart[] parts;
		if (length > getLength()){
			parts = new SnakePart[length+1];
			if (head == null)
				head = new SnakeHead(this);
			head.initLength(0, length+1, parts, pos, null);
		}
		else{
			return null;
		}
		return parts;
	}



/**
 * Function to return id. 
 */
	public int getID(){
		return id;
	}



/**
 * Function to print status.
 */
	public void getStatus(){
		System.out.println("Snake: ");
		System.out.println("\tID = " + id);
		switch (controller){
			case Enum.PLAYER1:  System.out.println("\tController = PLAYER1");  break;
			case Enum.PLAYER2:  System.out.println("\tController = PLAYER2");  break;
			case Enum.PLAYER3:  System.out.println("\tController = PLAYER3");  break;
			case Enum.PLAYER4:  System.out.println("\tController = PLAYER4");  break;
			case Enum.PLAYERAI: System.out.println("\tController = COMPUTER"); break;
			default:            System.out.println("\tController = PLAYER?");  break;
		}
		if (alive)
			System.out.println("\tLiving status = alive");
		else
			System.out.println("\tLiving status = dead");
		if (head != null)
			System.out.println("\tHead = 1");
		else
			System.out.println("\tHead = 0");
		System.out.println("\tBodies = " + (getLength()+1));
		System.out.println("\tNumber of stones = " + getNumOfStones());
		if (SnakeState.isNormal(state))
			System.out.println("\tState = NORMAL");
		if (SnakeState.isBerserked(state))
			System.out.println("\tState = BERSERKED(" + berserkTimeRemaining + ")");
		if (SnakeState.isFullOfStones(state))
			System.out.println("\tState = FULLOFSTONES");
	}



}
