/**
 * Class SnakeHead
 * Handles the Snakes' head as a separate entity.
 * Resolves collisions, and handles movement of the
 * Snake, and also eating Berrys.
 */
public class SnakeHead extends SnakePart {



/**
 * Class constructor with parameter.
 *
 * @param s Reference to the whole snake whose head it is.
 */
    public SnakeHead(Snake s) {
		id = idnum; idnum++;
		snake = s;
		type = Enum.SNAKEHEAD;
		hasStone = false;
		cell = null;
		nextBodyElement = null;
		prevBodyElement = null;
    }



/**
 * Responsoble for handling the collision with the next 
 * cell in the Field, regardless of the FieldCell's content.
 */    
    public void collide() {
		// Tells the neighbouring cell's content that this
		// SnakeHead is colliding with it.
		if (cell == null){
			return;
		}
		if (snake == null){
			return;
		}
		if (cell.getNeighbour(direction) == null){
			return;
		}
		(cell.getNeighbour(direction)).collide(snake);
    }



/**
 * Snake moves to the next field.
 */
	public void move(boolean grow){
		if (nextBodyElement != null)
			nextBodyElement.move(cell, grow);
		FieldCell tmp = cell.getNeighbour(direction);
		if (nextBodyElement == null){
			if (grow){
				nextBodyElement = new SnakeBody(snake);
				nextBodyElement.setPosition(cell);
				nextBodyElement.setPrev(this);
				nextBodyElement.setDirection(direction);
				cell.updateContent(nextBodyElement);
			}
			else
				cell.updateContent(new Empty());
		}
		cell = tmp;
		cell.updateContent(this);
	}



/**
 * Gets length
 */
	public int getLength(){
		int length = 0;
		if (nextBodyElement != null)
			return nextBodyElement.getLength(length);
		else return length;
	}



/**
 * Gets number of stones
 */
	public int getNumOfStones(){
		int stones = 0;
		if (hasStone)
			stones++;
		if (nextBodyElement != null)
			return nextBodyElement.getNumOfStones(stones);
		else return stones;
	}



}
